/*
 * This file is part of TechReborn, licensed under the MIT License (MIT).
 *
 * Copyright (c) 2017 TechReborn
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package techreborn.items.tools;

import net.minecraft.block.Block;
import net.minecraft.block.state.IBlockState;
import net.minecraft.creativetab.CreativeTabs;
import net.minecraft.entity.EntityLivingBase;
import net.minecraft.entity.item.EntityItem;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.init.Items;
import net.minecraft.item.ItemStack;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.NonNullList;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.World;
import net.minecraftforge.common.MinecraftForge;
import net.minecraftforge.event.world.BlockEvent;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;
import reborncore.common.powerSystem.PoweredItem;
import techreborn.config.ConfigTechReborn;
import techreborn.init.ModItems;

import java.util.ArrayList;
import java.util.List;

import net.minecraft.item.Item.ToolMaterial;

public class ItemAdvancedDrill extends ItemDrill {

	public ItemAdvancedDrill() {
		super(ToolMaterial.DIAMOND, "techreborn.advancedDrill", ConfigTechReborn.AdvancedDrillCharge,
			4.0F, 20F);
		this.cost = 250;
	}

	@SuppressWarnings({ "rawtypes", "unchecked" })
	@SideOnly(Side.CLIENT)
	@Override
	public void func_150895_a(CreativeTabs par2CreativeTabs, NonNullList itemList) {
		if(!func_194125_a(par2CreativeTabs)){
			return;
		}
		ItemStack stack = new ItemStack(ModItems.ADVANCED_DRILL);
		ItemStack uncharged = stack.func_77946_l();
		ItemStack charged = stack.func_77946_l();
		PoweredItem.setEnergy(getMaxPower(charged), charged);

		itemList.add(uncharged);
		itemList.add(charged);
	}

	@Override
	public boolean func_150897_b(IBlockState blockIn) {
		return Items.field_151046_w.func_150897_b(blockIn) || Items.field_151047_v.func_150897_b(blockIn);
	}

	@Override
	public boolean func_179218_a(ItemStack stack, World worldIn, IBlockState blockIn, BlockPos pos, EntityLivingBase entityLiving) {
		//TODO raytrace
		EnumFacing enumfacing = entityLiving.func_174811_aO().func_176734_d();
		if (entityLiving.field_70125_A < -50) {
			enumfacing = EnumFacing.DOWN;
		} else if (entityLiving.field_70125_A > 50) {
			enumfacing = EnumFacing.UP;
		}
		if (enumfacing == EnumFacing.SOUTH || enumfacing == EnumFacing.NORTH) {
			for (int i = -1; i < 2; i++) {
				for (int j = -1; j < 2; j++) {
					breakBlock(pos.func_177982_a(i, j, 0), stack, worldIn, entityLiving, pos);
				}
			}
		} else if (enumfacing == EnumFacing.EAST || enumfacing == EnumFacing.WEST) {
			for (int i = -1; i < 2; i++) {
				for (int j = -1; j < 2; j++) {
					breakBlock(pos.func_177982_a(0, j, i), stack, worldIn, entityLiving, pos);
				}
			}
		} else if (enumfacing == EnumFacing.DOWN || enumfacing == EnumFacing.UP) {
			for (int i = -1; i < 2; i++) {
				for (int j = -1; j < 2; j++) {
					breakBlock(pos.func_177982_a(j, 0, i), stack, worldIn, entityLiving, pos);
				}
			}
		}

		return super.func_179218_a(stack, worldIn, blockIn, pos, entityLiving);
	}

	public void breakBlock(BlockPos pos, ItemStack stack, World world, EntityLivingBase entityLiving, BlockPos oldPos) {
		if (oldPos == pos) {
			return;
		}
		if (!PoweredItem.canUseEnergy(cost, stack)) {
			return;
		}
		IBlockState blockState = world.func_180495_p(pos);
		Block block = blockState.func_177230_c();
		List<ItemStack> stuff = block.getDrops(world, pos, blockState, 0);
		List<ItemStack> dropList = new ArrayList<>();
		BlockEvent.HarvestDropsEvent event = new BlockEvent.HarvestDropsEvent(world, pos, blockState, 0, 1, dropList, (EntityPlayer) entityLiving, false);
		MinecraftForge.EVENT_BUS.post(event);
		for (ItemStack drop : dropList) {
			if (!drop.func_190926_b() && drop.func_190916_E() > 0) {
				stuff.add(drop);
			}
		}
		for (ItemStack drop : stuff) {
			if (world.field_72995_K) {
				continue;
			}
			final EntityItem entityitem = new EntityItem(world, oldPos.func_177958_n(), oldPos.func_177956_o(), oldPos.func_177952_p(), drop);
			entityitem.field_70159_w = (oldPos.func_177958_n() - oldPos.func_177958_n()) / 10.0f;
			entityitem.field_70181_x = 0.15000000596046448;
			entityitem.field_70179_y = (oldPos.func_177952_p() - oldPos.func_177952_p()) / 10.0f;
			world.func_72838_d(entityitem);
		}
		PoweredItem.useEnergy(cost, stack);
		world.func_175698_g(pos);
	}
}
